
/*
 *    File Name     : EciB1.cs
 *
 *    Description   : This sample shows how a mixed mode DLL can be used from
 *                    a C# application. The sample prompts for the name of a
 *                    defined CICS server and then launches a request on the
 *                    server. If the CICS server has security enabled, the
 *                    user is prompted for CICS login credentials.
 * 
 *                    This sample makes a non-extended synchronous ECI request,
 *                    and does not make use of callbacks, window handles or
 *                    event handles.
 *
 *    Prerequisites : This samples requires the CICS Program EC01 to be
 *                    installed on your CICS server for operation. The CICS
 *                    server must be set up to return the contents of the
 *                    commarea as ASCII text.
 *
 * Licensed Materials - Property of IBM
 * 
 * (c) Copyright IBM Corporation 2007, 2008
 * All Rights Reserved
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corporation
 *
 *         The following code is sample code created by IBM Corporation.  This
 *         sample code is not part of any standard IBM product and is provided
 *         to you solely for the purpose of assisting you in the development of
 *         your applications.  The code is provided 'AS IS', without warranty
 *         or condition of any kind.  IBM shall not be liable for any damages
 *         arising out of your use of the sample code, even if IBM has been
 *         advised of the possibility of such damages.
 */

using System;
using System.Text;
using EciWrapper;

namespace EciB1
{
    class Program
    {
        //Constants
        private const int   MAX_ECI_ATTEMPTS = 3;

        //Program entry point
        public static void Main(String[] cmdLine)
        {
            Program demoApp = new Program();
            demoApp.Start();
        }

        //Sends an ECI request and displays the returned commarea.
        public void Start()
        {
            CicsEci eciObj;
            string  serverName;

            //Display banner
            Console.WriteLine("EciWrapper ECI Sample");
            Console.WriteLine();

            //Prompt for a CICS server
            serverName = SelectServer();
        
            if (serverName == null) {
                return;
            }

            //Create the CicsEci object
            eciObj = new CicsEci();

            //Set properties for ECI call
            eciObj.ProgramName = "EC01";
            eciObj.SystemName = serverName;
            eciObj.Commarea = new byte[18];
            eciObj.CallType = EciCallType.Sync;
            eciObj.ExtendMode = EciExtendMode.NoExtend;

            if (AttemptEciCall(eciObj)) {
                //Display output
                Console.WriteLine();
                Console.WriteLine("Program " + eciObj.ProgramName +
                                  " returned with data:-");

                //Display commarea as hex
                Console.Write("       Hex: ");
                for (int i = 0; i < eciObj.Commarea.Length; i++)
                    Console.Write("{0:x02}", eciObj.Commarea[i]);
                Console.WriteLine();

                //Display commarea as ASCII
                Console.Write("ASCII text: ");                
                Console.WriteLine(Encoding.ASCII.GetString(eciObj.Commarea));
            }
        }

        //Queries the client daemon for a list of CICS servers then prompts the
        //user to select a server from the list.
        private string SelectServer()
        {
            CicsSystem[]    systems;
            string          strChoice;
            int             intChoice;

            //Obtain a list of CICS systems
            try {
                systems = CicsEci.ListSystems();
            } catch (EciException ex) {
                Console.WriteLine("ECI ListSystems failed with return code {0}",
                                  Enum.GetName(typeof(EciReturnCode), ex.ReturnCode));
                return null;
            }

            if (systems.Length == 0) {
                //No systems
                Console.WriteLine("No CICS servers have been defined");
                return null;
            } 

            intChoice = 0;
            while (intChoice == 0) {
                //Display systems list
                Console.WriteLine("CICS Servers Defined:");

                for (int i = 0; i < systems.Length; i++) {
                    Console.WriteLine("\t{0,3}. {1,-8} -{2}", (i + 1), systems[i].Name, systems[i].Description);
                }

                //Prompt for selection
                Console.WriteLine();
                Console.Write("Choose server to connect to, or q to quit: ");
                strChoice = Console.ReadLine();

                //Check for 'Q' (quit)
                if (strChoice.Equals("Q", StringComparison.OrdinalIgnoreCase)) {
                    return null;
                }

                //Parse number
                if (int.TryParse(strChoice, out intChoice)) {
                    //Check value is in range
                    if ((intChoice < 1) || (intChoice > systems.Length)) {
                        Console.WriteLine("Invalid selection");
                        Console.WriteLine();
                        intChoice = 0;
                    }

                } else {
                    //Not a valid number
                    Console.WriteLine("Invalid selection");
                    Console.WriteLine();
                }
            }

            //Return system name
            return systems[intChoice - 1].Name;
        }

        //Attempts to send the given ECI request, prompting the user to enter
        //CICS login credentials if required, up to a maximum of MAX_ECI_ATTEMPTS
        //attempts.
        private bool AttemptEciCall(CicsEci eciObj)
        {
            int attempts;

            attempts = 0;
            while (attempts < MAX_ECI_ATTEMPTS) {

                try {
                    //Attempt ECI call
                    eciObj.Execute();

                    //Call succeeded
                    return true;

                } catch (EciException ex) {

                    switch (ex.ReturnCode) {
                    case EciReturnCode.ErrSecurityError:

                        //Security error, prompt for login credentials
                        Console.Write("Enter your CICS User ID : ");
                        eciObj.UserID = Console.ReadLine();
                        Console.Write("Enter your CICS Password: ");
                        eciObj.Password = Console.ReadLine();
                        break;

                    case EciReturnCode.ErrTransactionAbend:

                        //Transaction abended
                        Console.WriteLine("The transaction abended: {0}", ex.AbendCode);
                        return false;

                    case EciReturnCode.ErrNoCics:

                        //CICS server not found
                        Console.WriteLine("CICS server not found");
                        return false;

                    default:
                        Console.WriteLine("ECI request failed with return code: {0}",
                                          Enum.GetName(typeof(EciReturnCode), ex.ReturnCode));
                        return false;
                    }
                }

                attempts++;
            }

            Console.WriteLine("Authentication failed");
            return false;
        }
    }
}
