
/*
 *     File Name     : CicsEci_2003.h
 *
 *     Description   : This sample shows how a mixed mode DLL can be used to
 *                     allow .NET applications to interact with CICS using the
 *                     CICS Transaction Gateway ECI API for C.
 *
 *                     This file is used by the CicsEci_2003.cpp source file
 *                     and declares the following managed types:
 *                          * EciCallType enumeration
 *                          * EciExtendMode enumeration
 *                          * EciReturnCode enumeration
 *                          * EciException class
 *                          * CicsSystem structure
 *                          * CicsEci class
 *
 *                     This file is the Managed C++ equivalent of CicsEci.h.
 *
 * Licensed Materials - Property of IBM
 * 
 * (c) Copyright IBM Corporation 2007, 2008
 * All Rights Reserved
 * 
 * US Government Users Restricted Rights - Use, duplication or
 * disclosure restricted by GSA ADP Schedule Contract with
 * IBM Corporation
 *
 *         The following code is sample code created by IBM Corporation.  This
 *         sample code is not part of any standard IBM product and is provided
 *         to you solely for the purpose of assisting you in the development of
 *         your applications.  The code is provided 'AS IS', without warranty
 *         or condition of any kind.  IBM shall not be liable for any damages
 *         arising out of your use of the sample code, even if IBM has been
 *         advised of the possibility of such damages.
 */

#pragma once

//The CICS_W32 macro must be defined before including cics_eci.h. It is used
//within cics_eci.h to define data types used by the CICS Transaction Gateway
//API.
#define CICS_W32
#include "cics_eci.h"

//Check that NULL has been defined.
#ifndef NULL
#define NULL    0
#endif

//This file can only be compiled by a compiler targetting the Common Language
//Runtime (CLR).
#ifndef _M_CEE              //_M_CEE
#error "Managed library requires /clr compiler option"
#else                       //_M_CEE

#ifndef __cplusplus_cli     //__cplusplus_cli

//Using Managed C++
#pragma message("Compiling as Managed C++")

//These are the namespaces we are using within the source files.
//The using namespace directive allows the types within the given namespace to
//be used without needing to specify the namespace every time.
using namespace System;
using namespace System::Runtime::InteropServices;


//The EciWrapper namespace contains the types defined in this file. A namespace
//is equivalent to a package in Java.
namespace EciWrapper {

    //The EciCallType enumeration groups the ECI constants that can be used
    //with the eci_call_type member of ECI_PARMS.
    public __value enum EciCallType : short
    {
        Sync =                  516, /* Synchronous call.                    */
        Async =                 517, /* Async call used with ECI_GET_REPLY.  */
        AsyncNotifyMsg =        518, /* Async call, notify by message.       */
        AsyncNotifySem =        519, /* Async call, notify by semaphore.     */
        GetReply =              520, /* Used to get reply to ASYNC call.     */
        GetReplyWait =          521, /* As above but wait for reply.         */
        GetSpecificReply =      528, /* Get specific ASYNC reply.            */
        GetSpecificReplyWait =  529, /* As above but wait for reply.         */
        StateSync =             522, /* Synchronous request for CICS status. */
        StateAsync =            523, /* As above but async, no notify.       */
        StateAsyncSem =         524, /* As above but notify by semaphore.    */
        StateAsyncMsg =         525, /* As above but notify by message.      */
    };

    //The EciExtendMode enumeration groups the ECI constants that can be used
    //with the eci_extend_mode member of ECI_PARMS.
    public __value enum EciExtendMode : short
    {
        NoExtend =              0,   /* These values are to be used with     */
        Extended =              1,   /* call_types other that ECI_GET_xxx &  */
        Commit =                2,   /* ECI_STATE_xxx.                       */
        Cancel =                Commit,
        Backout =               3,   /*                                      */
        StateImmediate =        4,   /* All ECI_STATE_xxx call-types.        */
        StateChanged =          5,   /* ECI_STATE_ASYNC_SEM &                */
        StateCancel =           6,   /* ECI_STATE_ASYNC_MSG call_types only. */
    };

    //Warning 4341 is generated when defining enumeration values that are less
    //than zero when the underlying enumeration type is smaller than int. It
    //can safely be ignored in this case.
#pragma warning( disable : 4341 )

    //The EciReturnCode enumeration groups the ECI constants that may be
    //returned by the CICS_ExternalCall function.
    public __value enum EciReturnCode : short
    {
        NoError =               0,
        ErrInvalidDataLength =  -1,
        ErrInvalidExtendMode =  -2,
        ErrNoCics =             -3,
        ErrCicsDied =           -4,
        ErrRequestTimeout =     -5,
        ErrNoReply =            ErrRequestTimeout,
        ErrResponseTimeout =    -6,
        ErrTransactionAbend =   -7,
        ErrExecNotResident =    -8,
        ErrLuwToken =           ErrExecNotResident,
        ErrSystemError =        -9,
        ErrNullWinHandle =      -10,
        ErrNullMessageID =      -12,
        ErrThreadCreateError =  -13,
        ErrInvalidCallType =    -14,
        ErrAlreadyActive =      -15,
        ErrResourceShortage =   -16,
        ErrNoSessions =         -17,
        ErrNullSemHandle =      -18,
        ErrInvalidDataArea =    -19,
        ErrInvalidVersion =     -21,
        ErrUnknownServer =      -22,
        ErrCallFromCallback =   -23,
        ErrMoreSystems =        -25,
        ErrNoSystems =          -26,
        ErrSecurityError =      -27,
        ErrMaxSystems =         -28,
        ErrMaxSessions =        -29,
        ErrRolledBack =         -30
    };
#pragma warning( default : 4341 )


    //The UnmanagedFunctionPointer attribute is applied to the following
    //delegate definition and denotes that when the delegate is converted into
    //a native function pointer, the generated function pointer should use the
    //cdecl calling convention.
    [UnmanagedFunctionPointer(CallingConvention::Cdecl)]
    public __delegate void EciCallback(short returnCode);


    //The EciException class is an exception class thrown when a non-zero
    //return code is returned from CICS_ExternalCall or CICS_EciListSystems.
    public __gc class EciException : public Exception
    {
    private:
        //Fields
        EciReturnCode   rc;     //The return code from the ECI call
        String __gc*    abCode; //The abend code

    public:
        //These properties are both read-only since they should not be changed
        //once the return code and abend code are set by the constructor.
        __property EciReturnCode get_ReturnCode();
        __property String* get_AbendCode();

        //Constructor
        EciException(short retCode, String* abendCode);
    };


    //The CicsSystem structure provides a managed equivalent of the
    //CICS_EciSystem_t type used by the CICS_EciListSystems call. An array of
    //CicsSystem is returned from the CicsEci::ListSystems method.
    public __value class CicsSystem
    {
    private:
        //Fields
        String __gc*    sysName;    //The name of the CICS system
        String __gc*    sysDesc;    //The description of the system

    public:
        //These properties are both read-only since they should not be changed
        //once the structure has been initialized by the constructor.
        __property String* get_Name();
        __property String* get_Description();

        //Constructor
        CicsSystem(String* name, String* desc);
    };


    //The CicsEci class provides a managed equivalent of the ECI_PARMS type of
    //the CICS Transaction Gateway API. It also wraps the CICS_ExternalCall and
    //CICS_EciListSystems function calls.
    public __gc class CicsEci
    {
    private:
        //Fields
        ECI_PARMS   __nogc*  eciParms;              //Pointer to unmanaged
                                                    //ECI_PARMS block
        Byte                 commareaArray __gc[];  //Managed byte array handle
        EciCallback __gc*    callbackDelegate;      //Managed delegate handle

        //Utility functions for converting between unmanaged and managed string
        //types.
        static void MgdStringToNtvChar(String* src, cics_char_t* dest, int len);
        static String* NtvCharToMgdString(cics_char_t* src, int len);

    public:
        //Each member of the ECI_PARMS structure is exposed as a property of
        //the CicsEci class, with the except eci_userid2 and eci_password2.
        //The eci_version member is set by the class constructor and the
        //eci_commarea_length member is set in the Execute method. The
        //reserved1 member does not need to be set.

        //eci_call_type
        __property EciCallType get_CallType();
        __property void set_CallType(EciCallType callType);

        //eci_program_name
        __property String* get_ProgramName();
        __property void set_ProgramName(String* programName);

        //eci_userid
        __property String* get_UserID();
        __property void set_UserID(String* userID);

        //eci_password
        __property String* get_Password();
        __property void set_Password(String* password);

        //eci_transid
        __property String* get_TransactionID();
        __property void set_TransactionID(String* transID);

        //eci_abend_code
        //The AbendCode property is read-only since it represents a returned
        //abend code and so should never be changed outside the class.
        __property String* get_AbendCode();

        //eci_commarea
        //This property also implicitly wraps the eci_commarea_length member
        //since all managed arrays have a Length property.
        __property Byte get_Commarea() __gc[];
        __property void set_Commarea(Byte commarea __gc[]);

        //eci_timeout
        __property short get_Timeout();
        __property void set_Timeout(short timeout);

        //eci_extend_mode
        __property EciExtendMode get_ExtendMode();
        __property void set_ExtendMode(EciExtendMode extendMode);

        //The eci_async_notify member of the ECI_PARMS structure is a union,
        //but since both union members are handles, they can be represented by
        //a single Handle property.
        //eci_async_notify
        __property IntPtr get_Handle();
        __property void set_Handle(IntPtr handle);

        //eci_message_id
        __property unsigned short get_MessageID();
        __property void set_MessageID(unsigned short messageID);

        //eci_message_qualifier
        __property short get_MessageQualifier();
        __property void set_MessageQualifier(short messageQualifier);

        //eci_luw_token
        __property int get_LUWToken();
        __property void set_LUWToken(int luwToken);

        //eci_sysid
        __property String* get_SysID();
        __property void set_SysID(String* sysID);

        //eci_system_name
        __property String* get_SystemName();
        __property void set_SystemName(String* systemName);

        //eci_callback
        __property EciCallback* get_Callback();
        __property void set_Callback(EciCallback* callback);

        //eci_tpn
        __property String* get_TPN();
        __property void set_TPN(String* tpn);

        //Constructor and destructor
        CicsEci();
        ~CicsEci();

        //The ListSystems method wraps the CICS_EciListSystems function. The
        //method is static since it does not use any of the instance members of
        //the CicsEci class.
        static CicsSystem ListSystems() __gc[];

        //The Execute method wraps the CICS_ExternalCall function.
        void Execute();
    };
}

#endif                      //__cplusplus_cli
#endif                      //_M_CEE